/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.intent;

import org.onosproject.nemo.model.common.UserId;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * Representation of language style request input object builder.
 */
public class LanguageStyleNemoRequestInputBuilder {

    private Map<Class<? extends LanguageStyleNemoRequestInput>,
            LanguageStyleNemoRequestInput> augmentation = new HashMap<>();

    private String nemoStatement;

    private UserId userId;

    /**
     * Creates a language style request input builder.
     */
    public LanguageStyleNemoRequestInputBuilder() {
    }

    /**
     * Creates a language style request input builder from
     * the language style request input.
     *
     * @param base language style request input object
     */
    public LanguageStyleNemoRequestInputBuilder(LanguageStyleNemoRequestInput base) {
        nemoStatement = base.getNemoStatement();
        userId = base.getUserId();
        if (base instanceof LanguageStyleNemoRequestInputImpl) {
            LanguageStyleNemoRequestInputImpl impl =
                    (LanguageStyleNemoRequestInputImpl) base;
            if (!impl.augmentation.isEmpty()) {
                augmentation = new HashMap<>(impl.augmentation);
            }
        }
    }

    /**
     * Sets the statement for the builder to use.
     *
     * @param statement statement
     * @return self
     */
    public LanguageStyleNemoRequestInputBuilder nemoStatement(String statement) {
        nemoStatement = statement;
        return this;
    }

    /**
     * Sets the user ID for the builder to use.
     *
     * @param id user ID
     * @return self
     */
    public LanguageStyleNemoRequestInputBuilder userId(UserId id) {
        userId = id;
        return this;
    }

    /**
     * Adds augmentation by a class type and a language style request input.
     * If the augmentation is null,remove the augmentation type key
     * from the hash map.
     *
     * @param augmentationType type that extends LanguageStyleNemoRequestInput
     * @param augmentation an language style request input object
     * @return self
     */
    public LanguageStyleNemoRequestInputBuilder addAugmentation(
            Class<? extends LanguageStyleNemoRequestInput> augmentationType,
            LanguageStyleNemoRequestInput augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Removes augmentation by a class type.
     *
     * @param augmentationType type that extends LanguageStyleNemoRequestInput
     * @return self
     */
    public LanguageStyleNemoRequestInputBuilder removeAugmentation(
            Class<? extends LanguageStyleNemoRequestInput> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a language style request input object.
     *
     * @return a language style request input object
     */
    public LanguageStyleNemoRequestInput build() {
        return new LanguageStyleNemoRequestInputImpl(this);
    }

    /**
     * Representation of the implement of LanguageStyleNemoRequestInput.
     */
    private static final class LanguageStyleNemoRequestInputImpl
            implements LanguageStyleNemoRequestInput {

        private final String nemoStatement;

        private final UserId userId;

        private final Map<Class<? extends LanguageStyleNemoRequestInput>,
                LanguageStyleNemoRequestInput> augmentation;

        private LanguageStyleNemoRequestInputImpl(
                LanguageStyleNemoRequestInputBuilder base) {
            nemoStatement = base.nemoStatement;
            userId = base.userId;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public String getNemoStatement() {
            return nemoStatement;
        }

        @Override
        public UserId getUserId() {
            return userId;
        }

        @Override
        public int hashCode() {
            return Objects.hash(nemoStatement, userId, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            LanguageStyleNemoRequestInputImpl other =
                    (LanguageStyleNemoRequestInputImpl) obj;
            return Objects.equals(nemoStatement, other.getNemoStatement()) &&
                    Objects.equals(userId, other.getUserId()) &&
                    Objects.equals(augmentation, other.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("nemoStatement", nemoStatement)
                    .add("userId", userId)
                    .add("augmentation", augmentation).toString();
        }
    }
}