/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.node.instance;

import org.onosproject.nemo.model.common.NodeId;

import com.google.common.collect.Maps;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * Representation of build sub node.
 */
public class SubNodeBuilder {

    private static final long MAX_LENGTH =  4294967295L;
    private static final long MIN_LENGTH =  0L;
    private static final String ERROR_LEN_MSG = "Invalid range:" +
            " %s, expected: [[" + MIN_LENGTH + "‥" + MAX_LENGTH + "]].";

    private Map<Class<? extends SubNode>, SubNode> augmentation =
            Maps.newHashMap();
    private SubNodeKey key;
    private long order;
    private NodeId nodeId;

    /**
     * Creates a new instance with default parameter.
     */
    public SubNodeBuilder() {
    }

    /**
     * Creates a new instance from a given base object.
     *
     * @param base sub node base
     */
    public SubNodeBuilder(SubNode base) {
        if (base.getKey() == null) {
            key = new SubNodeKey(base.getNodeId());
            base.getNodeId();
        } else {
            key = base.getKey();
            key.getNodeId();
        }
        order = base.getOrder();
    }

    /**
     * Sets the key to be used by the builder.
     *
     * @param key sub node key
     * @return self
     */
    public SubNodeBuilder key(SubNodeKey key) {
        this.key = key;
        return this;
    }

    /**
     * Sets the node id to be used by the builder.
     *
     * @param id node id
     * @return self
     */
    public SubNodeBuilder nodeId(NodeId id) {
        nodeId = id;
        return this;
    }

    /**
     * Sets the order id to be used by the builder.
     *
     * @param order the order
     * @return self
     */
    public SubNodeBuilder order(long order) {
        checkOrderRange(order);
        this.order = order;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public SubNodeBuilder addAugmentation(
            Class<SubNode> augmentationType, SubNode augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public SubNodeBuilder removeAugmentation(Class<SubNode> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds an sub node from this builder instance.
     *
     * @return the sub node
     */
    public SubNode build() {
        return new SubNodeImpl(this);
    }

    private static void checkOrderRange(long value) {
        if (value < MIN_LENGTH || value > MAX_LENGTH) {
            throw new IllegalArgumentException(
                    String.format(ERROR_LEN_MSG, value));
        }
    }

    /**
     * The implementation of sub node.
     */
    private static final class SubNodeImpl implements SubNode {

        private final SubNodeKey key;
        private final long order;
        private final NodeId nodeId;
        private final Map<Class<? extends SubNode>, SubNode> augmentation;

        private SubNodeImpl(SubNodeBuilder base) {
            if (base.key == null) {
                key = new SubNodeKey(base.nodeId);
                nodeId = base.nodeId;
            } else {
                key = base.key;
                nodeId = key.getNodeId();
            }
            order = base.order;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public NodeId getNodeId() {
            return nodeId;
        }

        @Override
        public long getOrder() {
            return order;
        }

        @Override
        public SubNodeKey getKey() {
            return key;
        }

        @Override
        public int hashCode() {
            return Objects.hash(key, nodeId, order, augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            SubNodeImpl o = (SubNodeImpl) obj;
            return Objects.equals(key, o.key) &&
                    Objects.equals(nodeId, o.nodeId) &&
                    Objects.equals(order, o.order) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("key", key)
                    .add("nodeId", nodeId)
                    .add("order", order)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}