/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.object.property.instance;

import static com.google.common.base.MoreObjects.toStringHelper;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import org.onosproject.nemo.model.object.property.instance.property.values.IntValue;
import org.onosproject.nemo.model.object.property.instance.property.values.RangeValue;
import org.onosproject.nemo.model.object.property.instance.property.values.StringValue;

import com.google.common.collect.Maps;

/**
 * The builder of property values.
 */
public class PropertyValuesBuilder {

    private Map<Class<? extends PropertyValues>, PropertyValues> augmentation =
            Maps.newHashMap();
    private List<IntValue> intValue;
    private List<StringValue> stringValue;
    private RangeValue rangeValue;

    /**
     * Creates a default user role builder.
     */
    public PropertyValuesBuilder() {
    }

    /**
     * Creates a property value builder from a property value object.
     *
     * @param base the property value object
     */
    public PropertyValuesBuilder(PropertyValues base) {
        intValue = base.getIntValues();
        rangeValue = base.getRangeValue();
        stringValue = base.getStringValues();
    }

    /**
     * Sets the integer values to be used by the builder.
     *
     * @param values property values
     * @return self
     */
    public PropertyValuesBuilder intValues(List<IntValue> values) {
        intValue = values;
        return this;
    }

    /**
     * Sets the range value to be used by the builder.
     *
     * @param value range value
     * @return self
     */
    public PropertyValuesBuilder rangeValue(RangeValue value) {
        rangeValue = value;
        return this;
    }

    /**
     * Sets the string values to be used by the builder.
     *
     * @param values string values
     * @return self
     */
    public PropertyValuesBuilder stringValues(List<StringValue> values) {
        stringValue = values;
        return this;
    }

    /**
     * Adds an augmentation to the augmentation list. If the given augmentation
     * is null, then removes the augmentation type from the map.
     *
     * @param augmentationType the type of argumentation
     * @param augmentation the augmentation
     * @return self
     */
    public PropertyValuesBuilder addAugmentation(
            Class<? extends PropertyValues> augmentationType,
            PropertyValues augmentation) {
        if (augmentation == null) {
            return removeAugmentation(augmentationType);
        }
        this.augmentation.put(augmentationType, augmentation);
        return this;
    }

    /**
     * Deletes an augmentation from the augmentation list.
     *
     * @param augmentationType the type of argumentation
     * @return self
     */
    public PropertyValuesBuilder removeAugmentation(
            Class<? extends PropertyValues> augmentationType) {
        augmentation.remove(augmentationType);
        return this;
    }

    /**
     * Builds a property value from this builder instance.
     *
     * @return property values
     */
    public PropertyValues build() {
        return new PropertyValuesImpl(this);
    }

    /**
     * The implementation of property value interface.
     */
    private static final class PropertyValuesImpl implements PropertyValues {

        private final List<IntValue> intValue;
        private final RangeValue rangeValue;
        private final List<StringValue> stringValue;
        private final Map<Class<? extends PropertyValues>, PropertyValues>
                augmentation;

        private PropertyValuesImpl(PropertyValuesBuilder base) {
            intValue = base.intValue;
            rangeValue = base.rangeValue;
            stringValue = base.stringValue;
            augmentation = new HashMap<>(base.augmentation);
        }

        @Override
        public List<IntValue> getIntValues() {
            return intValue;
        }

        @Override
        public RangeValue getRangeValue() {
            return rangeValue;
        }

        @Override
        public List<StringValue> getStringValues() {
            return stringValue;
        }

        @Override
        public int hashCode() {
            return Objects.hash(intValue, rangeValue, stringValue,
                    augmentation);
        }

        @Override
        public boolean equals(Object obj) {
            if (this == obj) {
                return true;
            }
            if (obj == null) {
                return false;
            }
            if (getClass() != obj.getClass()) {
                return false;
            }
            PropertyValuesImpl o = (PropertyValuesImpl) obj;
            return Objects.equals(intValue, o.intValue) &&
                    Objects.equals(rangeValue, o.rangeValue) &&
                    Objects.equals(stringValue, o.stringValue) &&
                    Objects.equals(augmentation, o.augmentation);
        }

        @Override
        public String toString() {
            return toStringHelper(this)
                    .add("intValue", intValue)
                    .add("rangeValue", rangeValue)
                    .add("stringValue", stringValue)
                    .add("augmentation", augmentation)
                    .toString();
        }
    }
}