/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.common;

import com.google.common.testing.EqualsTester;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;
import static org.onosproject.nemo.model.common.Description.E_BAD_LENGTH;
import static org.onosproject.nemo.model.common.Description.MAX_LENGTH;
import static org.onosproject.nemo.model.common.Description.MIN_LENGTH;

/**
 * Unit tests for {@link Description}.
 */
public class DescriptionTest extends AbstractCommonTest {

    private static final String DESC = "SomeDescription";
    private static final String OTHER = "OtherDescription";

    private static final int TOO_FEW = MIN_LENGTH - 1;
    private static final int TOO_MANY = MAX_LENGTH + 1;

    private static final String EXP_TOO_SMALL = String.format(E_BAD_LENGTH, TOO_FEW);
    private static final String EXP_TOO_LONG = String.format(E_BAD_LENGTH, TOO_MANY);

    private Description desc;
    private Description copy;
    private Description diff;

    @Test(expected = NullPointerException.class)
    public void constructWithNullString() {
        new Description((String) null);
    }

    @Test
    public void fromString() {
        desc = new Description(DESC);
        assertEquals("description not set", DESC, desc.getValue());
    }

    @Test(expected = NullPointerException.class)
    public void copyConstructWithNull() {
        new Description((Description) null);
    }

    @Test
    public void copyConstructor() {
        desc = new Description(DESC);
        copy = new Description(desc);
        new EqualsTester()
                .addEqualityGroup(desc, copy)
                .testEquals();
    }

    @Test
    public void nameTooSmall() {
        try {
            desc = new Description("");
            fail(E_NOEX);
        } catch (IllegalArgumentException e) {
            assertEquals(E_WREX, EXP_TOO_SMALL, e.getMessage());
        }
    }

    @Test
    public void nameTooLong() {
        try {
            StringBuilder temp = new StringBuilder();
            for (int i = 0; i < TOO_MANY; i++) {
                temp.append("a");
            }
            desc = new Description(temp.toString());
            fail(E_NOEX);
        } catch (IllegalArgumentException e) {
            assertEquals(E_WREX, EXP_TOO_LONG, e.getMessage());
        }
    }

    @Test
    public void equality() {
        desc = new Description(DESC);
        copy = new Description(DESC);
        diff = new Description(OTHER);
        new EqualsTester()
                .addEqualityGroup(desc, copy)
                .addEqualityGroup(diff)
                .testEquals();
    }
}