/*
 * Copyright 2016-present Open Networking Laboratory
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.onosproject.nemo.model.operation.action.definitions;

import static junit.framework.Assert.assertEquals;

import org.junit.Test;
import org.onosproject.nemo.model.common.ActionName;
import org.onosproject.nemo.model.operation.action.definitions.ActionDefinition
        .ParameterValueType;

import com.google.common.testing.EqualsTester;

/**
 * Unit tests for {@link ActionDefinitionBuilder}.
 */
public class ActionDefinitionBuilderTest {

    private static final String NAME = "SomeName";
    private static final String OTHER_NAME = "OtherName";
    private static final String NAME_ERROR_MSG = "name not set";
    private static final String KEY_ERROR_MSG = "key not set";
    private static final String TYPE_ERROR_MSG = "type not set";
    private static final int PARAMETER_VALUE_TYPE_VALID = 0;
    private static final int PARAMETER_VALUE_TYPE_NOT_VALID = 100;

    private ActionDefinitionKey key;
    private ActionDefinition.ParameterValueType type;
    private ActionName name;
    private ActionName otherName;
    private ActionDefinition listParam;
    private ActionDefinition def;
    private ActionDefinition copy;
    private ActionDefinition diff;

    @Test
    public void buildActionDefinitionWithActionDefinitionKey() {
        name = new ActionName(NAME);
        key = new ActionDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        listParam = new ActionDefinitionBuilder()
                .key(key)
                .build();
        def = new ActionDefinitionBuilder()
                .key(key)
                .parameterValueType(type)
                .addAugmentation(ActionDefinition.class, listParam)
                .build();
        def = new ActionDefinitionBuilder(def)
                .addAugmentation(ActionDefinition.class, null)
                .build();
        assertEquals(NAME_ERROR_MSG, name, def.getActionName());
        assertEquals(KEY_ERROR_MSG, key, def.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void buildActionDefinitionWithActionName() {
        name = new ActionName(NAME);
        key = new ActionDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        listParam = new ActionDefinitionBuilder()
                .actionName(name)
                .build();
        def = new ActionDefinitionBuilder()
                .parameterValueType(type)
                .actionName(name)
                .addAugmentation(ActionDefinition.class, listParam)
                .build();
        def = new ActionDefinitionBuilder(def)
                .addAugmentation(ActionDefinition.class, null)
                .build();
        assertEquals(NAME_ERROR_MSG, name, def.getActionName());
        assertEquals(KEY_ERROR_MSG, key, def.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void fromActionWithActionKey() {
        name = new ActionName(NAME);
        key = new ActionDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        def = new ActionDefinitionBuilder()
                .parameterValueType(type)
                .key(key)
                .build();
        copy = new ActionDefinitionBuilder(def).build();
        assertEquals(NAME_ERROR_MSG, name, copy.getActionName());
        assertEquals(KEY_ERROR_MSG, key, copy.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void fromActionWithActionName() {
        name = new ActionName(NAME);
        key = new ActionDefinitionKey(name);
        type = ParameterValueType.forValue(PARAMETER_VALUE_TYPE_VALID);
        def = new ActionDefinitionBuilder()
                .parameterValueType(type)
                .actionName(name)
                .build();
        copy = new ActionDefinitionBuilder(def).build();
        assertEquals(NAME_ERROR_MSG, name, copy.getActionName());
        assertEquals(KEY_ERROR_MSG, key, copy.getKey());
        assertEquals(TYPE_ERROR_MSG, type, def.getParameterValueType());
    }

    @Test
    public void equality() {
        name = new ActionName(NAME);
        def = new ActionDefinitionBuilder()
                .actionName(name)
                .build();
        copy = new ActionDefinitionBuilder()
                .actionName(name)
                .build();
        otherName = new ActionName(OTHER_NAME);
        diff = new ActionDefinitionBuilder()
                .actionName(otherName)
                .build();
        new EqualsTester()
                .addEqualityGroup(def, copy)
                .addEqualityGroup(diff)
                .testEquals();
    }

    @Test(expected = IllegalArgumentException.class)
    public void parameterValueTypeNotValid() {
        ParameterValueType.forValue(PARAMETER_VALUE_TYPE_NOT_VALID);
    }
}